# Daredevil medium module

This is the medium module for the Daredevil project. It's connected to one or more light modules and will receive light module data
from the CAN bus. This data will be propogated through Ethernet to be displayed on a console. See 'daredevil-visualizer'.

Currently this implementation only works on a Linux computer with Kvaser's CAN adapters. But will be ported to the Ultra96 dev board later.

New protocols and communication interfaces can be implemented using __'protocol_interface.rs'__ and 
__'communication_interface.rs'__. A communication interface needs to be made for the type of interface you
want to communicate over (e.g Ethernet, Serial etc.). Whereas a protocol interface is the implementation of
the message you want to use on the communication interface (e.g. CAN frames, Ethernet serialized etc.).

The medium currently uses two interfaces for communication and that is Ethernet and CAN. The implementations
of these two can be found in the source directory.

## Prerequisites
If running on a Linux device with Kvaser CAN hardware you need to install Kvaser drivers as well as CANlib. See [canlib-rs](https://gitlab.com/rust-daredevil-group/lib/canlib-rs).

## Installation

```
cd /to/your/directory/of/choice
git clone https://gitlab.com/rust-daredevil-group/daredevil-medium.git
```

## Usage
Setup the correct CAN bus settings,
```rust
// in main.rs 
let can_ch0 : Can<U128, UartSimpleMessage<U128>> = Can::new(); // using UartSimpleMessage as the protocol
let ch0_settings = can::CanBusSettings{...};
can_ch0.init(ch0_settings);
```
then initialize the ip address where the data will be sent to,
```rust
// in main.rs
let eth0 : Ethernet<U128, UartSimpleMessage<U128>> = Ethernet::new();
let ip_addr = String::from("127.0.0.1::7777");
eth0.init_stream(ip_addr);
```
After that just build and run it. 