pub extern crate typenum;
extern crate canlib_rs;

mod communication_interface;
mod protocol_interface;
mod uart;
mod ethernet;
mod can;
mod uart_simple_message;
mod ring_buffer;

use typenum::{U128};
use crate::communication_interface::CommunicationInterface;
use crate::ethernet::{Ethernet, EthernetTrait};
use crate::uart::UART;
use crate::can::{Can, CanTrait};
use crate::uart_simple_message::{UartSimpleMessage, UartSimpleMessageTrait};
use crate::canlib_rs::*;
use crate::ring_buffer::RingBufferInterface;
use std::{str,thread::sleep,time::Duration};
use aes_soft::Aes128;
use aes_soft::block_cipher_trait::generic_array::GenericArray as GenericArray;
use aes_soft::block_cipher_trait::BlockCipher;

fn main() {

    can_initialize_library(); // init CANlib which the Kvaser adapter uses

    // Create new communications
    //let mut u : UART<U128, UartSimpleMessage<U128>> = UART::new();
    //let mut v : Ethernet<U128, UartSimpleMessage<U128>> = Ethernet::new();
    let mut can_ch0 : Can<U128, UartSimpleMessage<U128>> = Can::new();
    //let ip_addr = String::from("130.240.200.93:7777");
    let running = true;

    // Prepare CAN bus settings
    let can_flags = CanOpen::CanFd as i32 | CanOpen::Exclusive as i32 | CanOpen::RequireExtended as i32; // Set channel flags
    let ch0_settings = can::CanBusSettings{ // Create CAN bus settings
        channel: 0, 
        driver_type: CanDriver::Normal, 
        freq: CanBitrate::_500K, 
        tseg1: 64, 
        tseg2: 16, 
        sjw: 16, 
        no_samp: 0,
        can_fd: true,
        flags: can_flags,
        freq_brs: CanBitrate::Fd2M,
        tseg1_brs: 15,
        tseg2_brs: 4,
        sjw_brs: 4,
    };

    // Initialize communications
    can_ch0.init(ch0_settings);
    //v.init_stream(ip_addr);

    while running {
        println!("Looping");
        let dur : std::time::Duration = Duration::new(0, 5000000); // sleeps because we don't want to read too much
        sleep(dur);
        
        can_ch0.rx(); // Listening on CAN bus
        
        let mut buffer = can_ch0.receive(); // Receive the data from CAN
        let ring_bfr = buffer.get_data(); // Get a RingBuffer of the data

        let mut arr1 = [0u8;16];
        let mut arr2 = [0u8;16];
        let mut arr3 = [0u8;16];
        let mut arr4 = [0u8;16];
        let length = ring_bfr.len();

        if ring_bfr.len() > 63 {
            println!("{}", length);
            for i in 0..16 {
                arr1[i as usize] = ring_bfr.pop();
            }
            for i in 0..16 {
                arr2[i as usize] = ring_bfr.pop();
            }
            for i in 0..16 {
                arr3[i as usize] = ring_bfr.pop();
            }
            for i in 0..16 {
                arr4[i as usize] = ring_bfr.pop();
            }

            let key = [0x2b,0x7e,0x15,0x16,0x28,0xae,0xd2,0xa6,0xab,0xf7,0x15,0x88,0x09,0xcf,0x4f,0x3c];
            let gen_key = GenericArray::from_slice(&key);
            let cipher = aes_soft::Aes128::new(&gen_key);

            let mut block1 = GenericArray::clone_from_slice(&arr1);
            cipher.decrypt_block(&mut block1);

            let mut block2 = GenericArray::clone_from_slice(&arr2);
            cipher.decrypt_block(&mut block2);

            let mut block3 = GenericArray::clone_from_slice(&arr3);
            cipher.decrypt_block(&mut block3);

            let mut block4 = GenericArray::clone_from_slice(&arr4);
            cipher.decrypt_block(&mut block4);

            let value1 : u8 = block1[0];
            let value2 : u8 = block1[1];
            let value3 : u8 = block1[2];
            let value4 : u8 = block1[3];
            let value5 : u8 = block1[4];
            let value6 : u8 = block1[5];
            let value7 : u8 = block1[6];
            let value8 : u8 = block1[7];

            /*for _ in 0..ring_bfr.len() {
                if set1 == false {
                    value1 = ring_bfr.pop() as u8;
                    set1 = true
                }
                else if set2 == false {
                    value2 = ring_bfr.pop() as u8;
                    set2 = true;
                }  else if set3 == false {
                    value3 = ring_bfr.pop() as u8;
                    set3 = true;
                }  else if set4 == false {
                    value4 = ring_bfr.pop() as u8;
                    set4 = true;
                }
            }*/

            let sensor1value : u32 = ((value2 as u32) | (value1 as u32) << 8  ).into();
            let sensor2value : u32 = ((value4 as u32) | (value3 as u32) << 8  ).into();
            let sensor3value : u32 = ((value6 as u32) | (value5 as u32) << 8  ).into();
            let sensor4value : u32 = ((value8 as u32) | (value7 as u32) << 8  ).into();
            println!("Sensor 1 value: {}", sensor1value);
            println!("Sensor 2 value: {}", sensor2value);
            println!("Sensor 3 value: {}", sensor3value);
            println!("Sensor 4 value: {}", sensor4value);
        }



        //send_value(value, &mut _v);
        

        /*println!("CAN string: {}", can_str);

        if can_str.contains("quit") {
            running = false;
        } else {
            send_str("Sending data now...", &mut _v);
            send_str(can_str.as_str(), &mut _v);
            send_str("Data transmission complete.", &mut _v);
        }*/
    }
    
    //send_str("Exiting loop.", &mut _v);
    println!("Done");

    can_ch0.terminate();
    println!("Bus terminated!");
    
    can_unload_library();

}

// Send str through Ethernet
fn send_str(s : &str, e: &mut Ethernet<U128, UartSimpleMessage<U128>>) {
    println!("{}", s);
    let msg1 : UartSimpleMessage<U128> = UartSimpleMessageTrait::new(s);
    e.send(msg1);
    e.tx();
}

// Sends u32 through Ethernet
fn send_value(s : u32, e: &mut Ethernet<U128, UartSimpleMessage<U128>>) {
    let msg1 : UartSimpleMessage<U128> = UartSimpleMessageTrait::new2(s);
    e.send(msg1);
    e.tx();
}

