extern crate generic_array;
extern crate heapless;

use crate::ring_buffer::RingBuffer;
use crate::ring_buffer::RingBufferInterface;
use crate::protocol_interface::ProtocolInterface;
use generic_array::ArrayLength;

pub struct UartSimpleMessage<N> where N: ArrayLength<u8> {
    data: RingBuffer<N>,
}

pub trait UartSimpleMessageTrait<N> where N: ArrayLength<u8>{
    fn new(s : &str) -> Self;
    fn new2(s : u32) -> Self;
    fn get_data(&mut self) -> &mut RingBuffer<N>;
}

impl<N> ProtocolInterface<N> for UartSimpleMessage<N> where N: ArrayLength<u8> {
    fn new() -> Self {
        UartSimpleMessage { data: RingBufferInterface::new() }
    }
    fn serialize(&mut self) -> &mut RingBuffer<N> {
        &mut self.data
    }
    fn deseralize(&mut self, r_buf: &mut RingBuffer<N>) {
        for _ in 0..r_buf.len() {
           self.data.push(r_buf.pop());
        }
    }
}

impl<N> UartSimpleMessageTrait<N> for UartSimpleMessage<N> where N: ArrayLength<u8> {
    fn new(s : &str) -> Self {
        let mut ret : UartSimpleMessage<N> = UartSimpleMessage { data: RingBufferInterface::new() };
        
        let ch = s.chars();

        for element in ch {
            ret.data.push(element as u8);
            println!("Adding char: {}", element);
        }

        ret
    }

    fn new2(s : u32) -> Self {
        let mut ret : UartSimpleMessage<N> = UartSimpleMessage { data: RingBufferInterface::new() };
        
        let s1 = (s >> 0) as u8;
        let s2 = (s >> 8) as u8;
        let s3 = (s >> 16) as u8;
        let s4 = (s >> 24) as u8;

        println!("{}", s1);
        println!("{}", s2);
        println!("{}", s3);
        println!("{}", s4);

        ret.data.push(s1);
        ret.data.push(s2);
        ret.data.push(s3);
        ret.data.push(s4);
        
        ret
    }

    fn get_data(&mut self) -> &mut RingBuffer<N> {
        &mut self.data
    }

}
